/*
 * Java port of Bullet (c) 2008 Martin Dvorak <jezek2@advel.cz>
 *
 * Bullet Continuous Collision Detection and Physics Library
 * Copyright (c) 2003-2007 Erwin Coumans  http://continuousphysics.com/Bullet/
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

package javabullet.demos.opengl;

import javabullet.linearmath.DebugDrawModes;
import javabullet.linearmath.IDebugDraw;
import javax.vecmath.Vector3f;
import static org.lwjgl.opengl.GL11.*;

/**
 *
 * @author jezek2
 */
public class GLDebugDrawer extends IDebugDraw {
	
	// JAVA NOTE: added
	private static final boolean DEBUG_NORMALS = false;
	
	private int debugMode;

	@Override
	public void drawLine(Vector3f from, Vector3f to, Vector3f color) {
		if (debugMode > 0) {
			glBegin(GL_LINES);
			glColor3f(color.x, color.y, color.z);
			glVertex3f(from.x, from.y, from.z);
			glVertex3f(to.x, to.y, to.z);
			glEnd();
		}
	}

	@Override
	public void setDebugMode(int debugMode) {
		this.debugMode = debugMode;
	}

	@Override
	public void draw3dText(Vector3f location, String textString) {
		glRasterPos3f(location.x,  location.y,  location.z);
		// TODO: BMF_DrawString(BMF_GetFont(BMF_kHelvetica10),textString);
	}

	@Override
	public void reportErrorWarning(String warningString) {
		System.err.println(warningString);
	}

	@Override
	public void drawContactPoint(Vector3f pointOnB, Vector3f normalOnB, float distance, int lifeTime, Vector3f color) {
		if ((debugMode & DebugDrawModes.DBG_DrawContactPoints.getMask()) != 0) {
			stack.vectors.push();
			try {
				Vector3f to = stack.vectors.get();
				to.scaleAdd(distance*100f, normalOnB, pointOnB);
				Vector3f from = pointOnB;

				// JAVA NOTE: added
				if (DEBUG_NORMALS) {
					to.normalize(normalOnB);
					to.scale(10f);
					to.add(pointOnB);
					glLineWidth(3f);
					glPointSize(6f);
					glBegin(GL_POINTS);
					glColor3f(color.x, color.y, color.z);
					glVertex3f(from.x, from.y, from.z);
					glEnd();
				}
				
				glBegin(GL_LINES);
				glColor3f(color.x, color.y, color.z);
				glVertex3f(from.x, from.y, from.z);
				glVertex3f(to.x, to.y, to.z);
				glEnd();
				
				// JAVA NOTE: added
				if (DEBUG_NORMALS) {
					glLineWidth(1f);
					glPointSize(1f);
				}

				glRasterPos3f(from.x, from.y, from.z);
				//char buf[12];
				//sprintf(buf," %d",lifeTime);
				// TODO: BMF_DrawString(BMF_GetFont(BMF_kHelvetica10),buf);
			}
			finally {
				stack.vectors.pop();
			}
		}
	}

	@Override
	public int getDebugMode() {
		return debugMode;
	}

}
